@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
@@ -28,11 +26,12 @@
 #include "gcr-certificate.h"
 #define DEBUG_FLAG GCR_DEBUG_CERTIFICATE_CHAIN
 #include "gcr-debug.h"
-#include "gcr-enum-types-base.h"
 #include "gcr-pkcs11-certificate.h"
 #include "gcr-simple-certificate.h"
 #include "gcr-trust.h"
 
+#include "gcr/gcr-enum-types-base.h"
+
 #include "egg/egg-error.h"
 
 /**
