@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
@@ -26,9 +24,10 @@
 #define DEBUG_FLAG GCR_DEBUG_GNUPG
 #include "gcr-debug.h"
 #include "gcr-gnupg-process.h"
-#include "gcr-marshal.h"
 #include "gcr-util.h"
 
+#include "gcr/gcr-marshal.h"
+
 #include <glib/gi18n-lib.h>
 
 #include <sys/wait.h>
