@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -24,14 +22,15 @@
 #include "gck/gck.h"
 
 #include "gcr-internal.h"
-#include "gcr-marshal.h"
-#include "gcr-oids.h"
 #include "gcr-openpgp.h"
 #include "gcr-openssh.h"
 #include "gcr-parser.h"
 #include "gcr-record.h"
 #include "gcr-types.h"
 
+#include "gcr/gcr-marshal.h"
+#include "gcr/gcr-oids.h"
+
 #include "egg/egg-armor.h"
 #include "egg/egg-asn1x.h"
 #include "egg/egg-asn1-defs.h"
@@ -59,7 +58,7 @@
  * connected to. Data is then fed to the parser via gcr_parser_parse_data()
  * or gcr_parser_parse_stream().
  *
- * During the #GcrParsed::parsed signal the attributes that make up the currently
+ * During the #GcrParser::parsed signal the attributes that make up the currently
  * parsed item can be retrieved using the gcr_parser_get_parsed_attributes()
  * function.
  */
@@ -127,6 +126,7 @@ struct _GcrParsed {
 	GBytes *data;
 	gboolean sensitive;
 	GcrDataFormat format;
+	gchar *filename;
 	struct _GcrParsed *next;
 };
 
@@ -135,6 +135,7 @@ struct _GcrParserPrivate {
 	gboolean normal_formats;
 	GPtrArray *passwords;
 	GcrParsed *parsed;
+	gchar *filename;
 };
 
 G_DEFINE_TYPE (GcrParser, gcr_parser, G_TYPE_OBJECT);
@@ -380,24 +381,31 @@ push_parsed (GcrParser *self,
 	parsed->refs = 0;
 	parsed->sensitive = sensitive;
 	parsed->next = self->pv->parsed;
+	parsed->filename = g_strdup (gcr_parser_get_filename (self));
 	self->pv->parsed = parsed;
 	return parsed;
 }
 
+static void
+_gcr_parsed_free (GcrParsed *parsed)
+{
+	gck_builder_clear (&parsed->builder);
+	if (parsed->attrs)
+		gck_attributes_unref (parsed->attrs);
+	if (parsed->data)
+		g_bytes_unref (parsed->data);
+	g_free (parsed->label);
+	g_free (parsed->filename);
+	g_free (parsed);
+}
+
 static void
 pop_parsed (GcrParser *self,
             GcrParsed *parsed)
 {
 	g_assert (parsed == self->pv->parsed);
-
 	self->pv->parsed = parsed->next;
-
-	gck_builder_clear (&parsed->builder);
-	gck_attributes_unref (parsed->attrs);
-	if (parsed->data)
-		g_bytes_unref (parsed->data);
-	g_free (parsed->label);
-	g_free (parsed);
+	_gcr_parsed_free (parsed);
 }
 
 static gint
@@ -1620,7 +1628,6 @@ parse_base64_spkac (GcrParser *self,
 	GBytes *bytes;
 	gsize n_data;
 	gint ret;
-
 	data = g_bytes_get_data (dat, &n_data);
 
 	if (n_data > PREFIX_LEN && memcmp (PREFIX, data, PREFIX_LEN))
@@ -2370,10 +2377,9 @@ gcr_parser_add_password (GcrParser *self, const gchar *password)
 }
 
 /**
- * gcr_parser_parse_data:
+ * gcr_parser_parse_bytes:
  * @self: The parser
- * @data: (array length=n_data): the data to parse
- * @n_data: The length of the data
+ * @data: the data to parse
  * @error: A location to raise an error on failure.
  *
  * Parse the data. The #GcrParser::parsed and #GcrParser::authenticate signals
@@ -2382,22 +2388,20 @@ gcr_parser_add_password (GcrParser *self, const gchar *password)
  * Returns: Whether the data was parsed successfully or not.
  */
 gboolean
-gcr_parser_parse_data (GcrParser *self,
-                       const guchar *data,
-                       gsize n_data,
-                       GError **error)
+gcr_parser_parse_bytes (GcrParser *self,
+                        GBytes *data,
+                        GError **error)
 {
 	ForeachArgs args = { self, NULL, GCR_ERROR_UNRECOGNIZED };
 	const gchar *message = NULL;
 	gint i;
 
 	g_return_val_if_fail (GCR_IS_PARSER (self), FALSE);
-	g_return_val_if_fail (data || !n_data, FALSE);
+	g_return_val_if_fail (data != NULL, FALSE);
 	g_return_val_if_fail (!error || !*error, FALSE);
 
-	if (data && n_data) {
-		/* TODO: Once GBytes is in the API, copy here */
-		args.data = g_bytes_new_static (data, n_data);
+	if (g_bytes_get_size (data) > 0) {
+		args.data = g_bytes_ref (data);
 
 		/* Just the specific formats requested */
 		if (self->pv->specific_formats) {
@@ -2439,6 +2443,40 @@ gcr_parser_parse_data (GcrParser *self,
 	return FALSE;
 }
 
+/**
+ * gcr_parser_parse_data:
+ * @self: The parser
+ * @data: (array length=n_data): the data to parse
+ * @n_data: The length of the data
+ * @error: A location to raise an error on failure.
+ *
+ * Parse the data. The #GcrParser::parsed and #GcrParser::authenticate signals
+ * may fire during the parsing.
+ *
+ * A copy of the data will be made. Use gcr_parser_parse_bytes() to avoid this.
+ *
+ * Returns: Whether the data was parsed successfully or not.
+ */
+gboolean
+gcr_parser_parse_data (GcrParser *self,
+                       const guchar *data,
+                       gsize n_data,
+                       GError **error)
+{
+	GBytes *bytes;
+	gboolean ret;
+
+	g_return_val_if_fail (GCR_IS_PARSER (self), FALSE);
+	g_return_val_if_fail (data || !n_data, FALSE);
+	g_return_val_if_fail (!error || !*error, FALSE);
+
+	bytes = g_bytes_new (data, n_data);
+	ret = gcr_parser_parse_bytes (self, bytes, error);
+	g_bytes_unref (bytes);
+
+	return ret;
+}
+
 /**
  * gcr_parser_format_enable:
  * @self: The parser
@@ -2551,6 +2589,37 @@ gcr_parsed_get_type (void)
 	return type;
 }
 
+/**
+ * gcr_parser_get_filename:
+ * @self: a parser item
+ *
+ * Get the filename of the parser item.
+ *
+ * Returns: the filename set on the parser, or %NULL
+ */
+const gchar *
+gcr_parser_get_filename (GcrParser *self)
+{
+	g_return_val_if_fail (GCR_IS_PARSER (self), NULL);
+	return self->pv->filename;
+}
+
+/**
+ * gcr_parser_set_filename:
+ * @self: a parser item
+ * @filename: (allow-none): a string of the filename of the parser item
+ *
+ * Sets the filename of the parser item.
+ */
+void
+gcr_parser_set_filename (GcrParser *self,
+                         const gchar *filename)
+{
+	g_return_if_fail (GCR_IS_PARSER (self));
+	g_free (self->pv->filename);
+	self->pv->filename = g_strdup (filename);
+}
+
 /**
  * gcr_parsed_ref:
  * @parsed: a parsed item
@@ -2575,6 +2644,7 @@ gcr_parsed_ref (GcrParsed *parsed)
 	copy = g_new0 (GcrParsed, 1);
 	copy->refs = 1;
 	copy->label = g_strdup (gcr_parsed_get_label (parsed));
+	copy->filename = g_strdup (gcr_parsed_get_filename (parsed));
 	copy->attrs = gcr_parsed_get_attributes (parsed);
 	copy->format = gcr_parsed_get_format (parsed);
 	if (copy->attrs)
@@ -2609,13 +2679,7 @@ gcr_parsed_unref (gpointer parsed)
 	g_return_if_fail (parsed != NULL);
 
 	if (g_atomic_int_dec_and_test (&par->refs)) {
-		gck_builder_clear (&par->builder);
-		if (par->attrs)
-			gck_attributes_unref (par->attrs);
-		g_free (par->label);
-		if (par->data)
-			g_bytes_unref (par->data);
-		g_free (par);
+		_gcr_parsed_free (parsed);
 	}
 }
 
@@ -2760,6 +2824,22 @@ gcr_parser_get_parsed_block (GcrParser *self,
 	return gcr_parsed_get_data (self->pv->parsed, n_block);
 }
 
+
+/**
+ * gcr_parser_get_parsed_bytes:
+ * @self: a parser
+ *
+ * Get the raw data block that represents this parsed object. This is only
+ * valid during the #GcrParser::parsed signal.
+ *
+ * Returns: (transfer none): the raw data block of the currently parsed item
+ */
+GBytes *
+gcr_parser_get_parsed_bytes (GcrParser *self)
+{
+	return gcr_parsed_get_bytes (self->pv->parsed);
+}
+
 /**
  * gcr_parsed_get_data:
  * @parsed: a parsed item
@@ -2774,15 +2854,36 @@ const guchar *
 gcr_parsed_get_data (GcrParsed *parsed,
                      gsize *n_data)
 {
+	GBytes *bytes;
+
 	g_return_val_if_fail (n_data != NULL, NULL);
 
+	bytes = gcr_parsed_get_bytes (parsed);
+	if (bytes == NULL) {
+		*n_data = 0;
+		return NULL;
+	}
+
+	return g_bytes_get_data (bytes, n_data);
+}
+
+/**
+ * gcr_parsed_get_bytes:
+ * @parsed: a parsed item
+ *
+ * Get the raw data block for the parsed item.
+ *
+ * Returns: (transfer none): the raw data of the parsed item, or %NULL
+ */
+GBytes *
+gcr_parsed_get_bytes (GcrParsed *parsed)
+{
 	while (parsed != NULL) {
 		if (parsed->data != NULL)
-			return g_bytes_get_data (parsed->data, n_data);
+			return parsed->data;
 		parsed = parsed->next;
 	}
 
-	*n_data = 0;
 	return NULL;
 }
 
@@ -2825,6 +2926,22 @@ gcr_parsed_get_format (GcrParsed *parsed)
 
 	return 0;
 }
+
+/**
+ * gcr_parsed_get_filename:
+ * @parsed: a parsed item
+ *
+ * Get the filename of the parsed item.
+ *
+ * Returns: (transfer none): the filename of
+ *          the parsed item, or %NULL
+ */
+const gchar *
+gcr_parsed_get_filename (GcrParsed *parsed)
+{
+	g_return_val_if_fail (parsed != NULL, NULL);
+	return parsed->filename;
+}
 /* ---------------------------------------------------------------------------------
  * STREAM PARSING
  */
@@ -2920,12 +3037,16 @@ static void
 state_parse_buffer (GcrParsing *self, gboolean async)
 {
 	GError *error = NULL;
+	GBytes *bytes;
 	gboolean ret;
 
 	g_assert (GCR_IS_PARSING (self));
 	g_assert (self->buffer);
 
-	ret = gcr_parser_parse_data (self->parser, self->buffer->data, self->buffer->len, &error);
+	bytes = g_byte_array_free_to_bytes (self->buffer);
+	self->buffer = NULL;
+	ret = gcr_parser_parse_bytes (self->parser, bytes, &error);
+	g_bytes_unref (bytes);
 
 	if (ret == TRUE) {
 		next_state (self, state_complete);
