@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
@@ -340,15 +338,13 @@ perform_add_pinned_certificate (GckAttributes *search,
 	en = gck_slots_enumerate_objects (slots, search, CKF_RW_SESSION);
 	gck_list_unref_free (slots);
 
-	/* We need an error below */
-	if (error && !*error)
-		*error = lerr;
-
-	object = gck_enumerator_next (en, cancellable, error);
+	object = gck_enumerator_next (en, cancellable, &lerr);
 	g_object_unref (en);
 
-	if (*error)
+	if (lerr != NULL) {
+		g_propagate_error (error, lerr);
 		return FALSE;
+	}
 
 	/* It already exists */
 	if (object) {
@@ -363,17 +359,17 @@ perform_add_pinned_certificate (GckAttributes *search,
 	/* Find an appropriate token */
 	slot = gcr_pkcs11_get_trust_store_slot ();
 	if (slot == NULL) {
-		g_set_error (error, GCK_ERROR, CKR_FUNCTION_FAILED,
+		g_set_error (&lerr, GCK_ERROR, CKR_FUNCTION_FAILED,
 		             /* Translators: A pinned certificate is an exception which
 		                trusts a given certificate explicitly for a purpose and
 		                communication with a certain peer. */
 		             _("Couldn't find a place to store the pinned certificate"));
 		ret = FALSE;
 	} else {
-		session = gck_slot_open_session (slot, CKF_RW_SESSION, NULL, error);
+		session = gck_slot_open_session (slot, CKF_RW_SESSION, NULL, &lerr);
 		if (session != NULL) {
 			object = gck_session_create_object (session, gck_builder_end (&builder),
-			                                    cancellable, error);
+			                                    cancellable, &lerr);
 			if (object != NULL) {
 				g_object_unref (object);
 				ret = TRUE;
@@ -387,8 +383,8 @@ perform_add_pinned_certificate (GckAttributes *search,
 
 	gck_builder_clear (&builder);
 
-	/* Our own local error pointer */
-	g_clear_error (&lerr);
+	if (!ret)
+		g_propagate_error (error, lerr);
 
 	return ret;
 }
