@@ -14,9 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * License along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Stef Walter <stefw@collabora.co.uk>
  */
@@ -27,7 +25,9 @@
 #include "gcr/gcr-library.h"
 #include "gcr/gcr-marshal.h"
 #include "gcr/gcr-parser.h"
+#include "gcr/gcr-version.h"
 
+#include "eggimagemenuitem.h"
 #include "gcr-import-button.h"
 #include "gcr-pkcs11-import-interaction.h"
 
@@ -87,6 +87,10 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 static GQuark QUARK_IMPORTER = 0;
 
+#if GCR_CHECK_VERSION(4,0,0)
+#error Port this class to derive from GtkMenuButton during 4.x ABI bump
+#endif
+
 G_DEFINE_TYPE (GcrImportButton, gcr_import_button, GTK_TYPE_BUTTON);
 
 static void
@@ -372,12 +376,12 @@ update_importer_menu (GcrImportButton *self)
 
 	for (l = self->pv->importers; l != NULL; l = g_list_next (l)) {
 		g_object_get (l->data, "label", &label, "icon", &icon, NULL);
-		menu_item = gtk_image_menu_item_new_with_label (label);
+		menu_item = egg_image_menu_item_new_with_label (label);
 		g_signal_connect (menu_item, "activate", G_CALLBACK (on_importer_menu_activated), self);
 		g_object_set_qdata (G_OBJECT (menu_item), QUARK_IMPORTER, l->data);
 		image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item), image);
-		gtk_image_menu_item_set_always_show_image (GTK_IMAGE_MENU_ITEM (menu_item), TRUE);
+		egg_image_menu_item_set_image (EGG_IMAGE_MENU_ITEM (menu_item), image);
+		egg_image_menu_item_set_always_show_image (EGG_IMAGE_MENU_ITEM (menu_item), TRUE);
 		gtk_widget_show (image);
 		gtk_widget_show (menu_item);
 		gtk_container_add (GTK_CONTAINER (self->pv->menu), menu_item);
